=== modified file 'apt-pkg/contrib/mmap.cc'
--- apt-pkg/contrib/mmap.cc	2012-05-21 17:12:25 +0000
+++ apt-pkg/contrib/mmap.cc	2012-06-16 12:11:16 +0000
@@ -248,12 +248,6 @@ DynamicMMap::DynamicMMap(unsigned long F
 	if (Grow == 0)
 		this->Flags &= ~Moveable;
 
-#ifndef __linux__
-	// kfreebsd doesn't have mremap, so we use the fallback
-	if ((this->Flags & Moveable) == Moveable)
-		this->Flags |= Fallback;
-#endif
-
 #ifdef _POSIX_MAPPED_FILES
 	if ((this->Flags & Fallback) != Fallback) {
 		// Set the permissions.
@@ -454,6 +448,18 @@ bool DynamicMMap::Grow() {
 
 		if(Base == MAP_FAILED)
 			return false;
+#elif  defined(_POSIX_MAPPED_FILES)
+		if (Fd != 0 && (Flags & Moveable) && SyncToFd == 0) {
+			/* Poor man's mremap. We need to restore iSize afterwards,
+			* as that is set by RawAllocate in DynamicMMap */
+			unsigned long iSizeO = iSize;
+			munmap(Base, WorkSpace);
+			if (Map(*Fd) == false)
+				return false;
+			iSize = iSizeO;
+		} else {
+			return false;
+		}
 #else
 		return false;
 #endif

